@@ -1,5 +1,5 @@
 """
-Import tiddlers, Cook recipes, TiddlyWikis into TiddlyWeb.
+Import tiddlers, Cook recipes,  and TiddlyWikis into TiddlyWeb.
 
 If you wish to use the provided "twimport" twanager command
 you must add to twanager_plugins in tiddlywebconfig.py:
@@ -7,6 +7,36 @@ you must add to twanager_plugins in tiddlywebconfig.py:
     config = {
         'twanager_plugins': ['tiddlywebplugins.twimport'],
     }
+
+twimport has two primary purposes:
+
+    * Using the "twanager twimport" command to import content into a
+      running instance.
+    * Using the functionality in the plugin to help build instance
+      packages using tiddler content stored in many locations.
+
+This latter functionality relies on functionality provided by
+"tiddlywebplugins.ibuiler" and uses a modified form of the cook
+recipe format. Tiddlers may be referred to with either a local
+file path, or a remote URI and have an optional mime type:
+
+    tiddler: <uri> [mime type]
+
+The mime type option is provided to allow setting the mime type
+of the stored tiddler. If no type is provided the system will
+attempt to determine the mime based on HTTP response headers 
+or by guessing from the file extension.
+
+If a tiddler uri ends in '.js' it will be processed as a
+TiddlyWiki plugin (and, in the absence of a .meta file, get a
+systemConfig tag). To avoid this behavior set the mime type
+explicitly.
+
+When a tiddler is loaded, the code will look for a meta file
+in the same location. This optional file contains meta
+information about the tiddler, such as tags, modifier, and
+fields. The format is the same as the top section of a .tid
+file.
 """
 
 import os
@@ -19,14 +49,15 @@ from html5lib import HTMLParser, treebuilders
 from tiddlyweb.model.tiddler import Tiddler, string_to_tags_list
 from tiddlyweb.serializer import Serializer
 from tiddlyweb.manage import make_command
+from tiddlyweb.util import pseudo_binary
 
 from tiddlywebplugins.utils import get_store
 
 ACCEPTED_RECIPE_TYPES = ['tiddler', 'plugin', 'recipe']
 ACCEPTED_TIDDLER_TYPES = ['js', 'tid', 'tiddler']
 COOK_VARIABLES = {
-    'TW_TRUNKDIR': 'http://svn.tiddlywiki.org/Trunk',
-    'TW_ROOT': 'http://svn.tiddlywiki.org/Trunk'
+        'TW_TRUNKDIR': 'https://raw.github.com/TiddlyWiki/tiddlywiki/master',
+        'TW_ROOT': 'https://raw.github.com/TiddlyWiki/tiddlywiki/master',
 }
 
 
@@ -46,15 +77,19 @@ def init(config):
 
 
 def import_list(bag_name, urls, store):
-    """Import a list of URIs into bag."""
+    """
+    Import a list of URIs into the named bag.
+    """
     for url in urls:
         import_one(bag_name, url, store)
 
 
 def import_one(bag_name, url, store):
-    """Import one URI into bag. If the URI has a #fragment it
+    """
+    Import one URI into bag. If the URI has a #fragment it
     will be processed as a TiddlyWiki permaview fragment and
-    used to limit the tiddlers that get saved."""
+    used to limit the tiddlers that get saved.
+    """
     fragments = []
     if '#' in url:
         url, fragment = url.split('#', 1)
@@ -76,7 +111,7 @@ def import_one(bag_name, url, store):
 
 def recipe_to_urls(url):
     """
-    Provided a url or path to a recipe, explode the recipe to
+    Provided a url or path to a Cook-style recipe, explode the recipe to
     a list of URLs of tiddlers (of various types).
     """
     url, handle = get_url_handle(url)
@@ -88,9 +123,14 @@ def url_to_tiddler(url):
     Given a url to a tiddlers of some form,
     return a Tiddler object.
     """
+    mime_type = None
+    if ' ' in url:
+        uri, mime_type = url.split(' ', 1)
+        if '/' in mime_type:
+            url = uri
     url, handle = get_url_handle(url)
 
-    if url.endswith('.js'):
+    if url.endswith('.js') and not mime_type:
         tiddler = from_plugin(url, handle)
     elif url.endswith('.tid'):
         tiddler = from_tid(url, handle)
@@ -98,13 +138,14 @@ def url_to_tiddler(url):
         tiddler = from_tiddler(handle)
     else:
         # binary tiddler
-        tiddler = from_special(url, handle)
+        tiddler = from_special(url, handle, mime=mime_type)
     return tiddler
 
 
 def wiki_to_tiddlers(url):
     """
-    Retrieve a .wiki or .html and extract the contained tiddlers.
+    Retrieve a .wiki or .html as a TiddlyWiki and extract the
+    contained tiddlers.
     """
     url, handle = get_url_handle(url)
     return wiki_string_to_tiddlers(handle.read().decode('utf-8', 'replace'))
@@ -112,7 +153,7 @@ def wiki_to_tiddlers(url):
 
 def wiki_string_to_tiddlers(content):
     """
-    Turn a string that is a wiki into tiddler.
+    Turn a string that is a TiddlyWiki into individual tiddlers.
     """
     parser = HTMLParser(tree=treebuilders.getTreeBuilder('dom'))
     doc = parser.parse(content)
@@ -139,7 +180,7 @@ def wiki_string_to_tiddlers(content):
 
 def from_plugin(uri, handle):
     """
-    generates Tiddler from a JavaScript (and accompanying meta) file
+    Generate a tiddler from a JavaScript (and accompanying meta) file
     If there is no .meta file, title and tags assume default values.
     """
     default_title = _get_title_from_uri(uri)
@@ -166,12 +207,20 @@ def from_plugin(uri, handle):
     return _from_text(title, tiddler_text)
 
 
-def from_special(uri, handle):
+def from_special(uri, handle, mime=None):
     """
-    This is borrowed from ben G's bimport.
+    Import a binary or pseudo binary tiddler. If a mime is provided,
+    set the type of the tiddler to that. Otherwise use the type determined
+    by the URL handler. If a meta file is present and has a type, it will
+    be used.
+
+    This code is inspired by @bengillies bimport.
     """
     title = _get_title_from_uri(uri)
-    content_type = handle.headers.type
+    if mime:
+        content_type = mime
+    else:
+        content_type = handle.headers.type
     data = handle.read()
 
     meta_uri = '%s.meta' % uri
@@ -181,10 +230,12 @@ def from_special(uri, handle):
     except (urllib2.HTTPError, urllib2.URLError, IOError, OSError):
         tiddler = Tiddler(title)
 
-    if content_type.startswith('text/'):
+    if not tiddler.type and content_type:
+        tiddler.type = content_type
+
+    if pseudo_binary(content_type):
         data = data.decode('utf-8', 'ignore')
 
-    tiddler.type = content_type
     tiddler.text = data
 
     return tiddler
@@ -192,7 +243,7 @@ def from_special(uri, handle):
 
 def from_tid(uri, handle):
     """
-    generates Tiddler from a TiddlyWeb-style .tid file
+    generates a tiddler from a TiddlyWeb-style .tid file
     """
     title = _get_title_from_uri(uri)
     return _from_text(title, handle.read().decode('utf-8', 'replace'))
@@ -200,7 +251,7 @@ def from_tid(uri, handle):
 
 def from_tiddler(handle):
     """
-    generates Tiddler from a Cook-style .tiddler file
+    generates a tiddler from a Cook-style .tiddler file
     """
     content = handle.read().decode('utf-8', 'replace')
     content = _escape_brackets(content)
@@ -293,7 +344,7 @@ def _get_url(url):
 
 def _from_text(title, content):
     """
-    generates Tiddler from an RFC822-style string
+    Generates a tiddler from an RFC822-style string
 
     This corresponds to TiddlyWeb's text serialization of TiddlerS.
     """
